@@ -1,3 +1,25 @@
+//
+// fetwidgetuiform.cpp
+//
+// Copyright 2014-2015 by John Pietrzak  (jpietrzak8@gmail.com)
+//
+// This file is part of Fettuccine.
+//
+// Fettuccine is free software; you can redistribute it and/or
+// modify it under the terms of the GNU General Public License
+// as published by the Free Software Foundation; either version 2
+// of the License, or (at your option) any later version.
+//
+// Fettuccine is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Fettuccine; if not, write to the Free Software Foundation,
+// Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+//
+
 #include "fetwidgetuiform.h"
 #include "ui_fetwidgetuiform.h"
 
@@ -5,6 +27,7 @@
 #include "fetwidgetsettingsdialog.h"
 #include <QResizeEvent>
 #include <QFontMetrics>
+#include <QSettings>
 
 
 FetWidgetUIForm::FetWidgetUIForm(
@@ -12,7 +35,9 @@ FetWidgetUIForm::FetWidgetUIForm(
   : QWidget(parent),
     ui(new Ui::FetWidgetUIForm),
     webcamImage(0),
-    settingsDialog(0)
+    settingsDialog(0),
+    width(320),
+    height(200)
 {
   ui->setupUi(this);
 
@@ -20,6 +45,19 @@ FetWidgetUIForm::FetWidgetUIForm(
   ui->nextButton->hide();
   ui->prevButton->hide();
   ui->nameLabel->hide();
+  ui->statusButton->hide();
+
+  QSettings settings("pietrzak.org", "Fettuccine");
+
+  if (settings.contains("WidgetWidth"))
+  {
+    width = settings.value("WidgetWidth").toInt();
+  }
+
+  if (settings.contains("WidgetHeight"))
+  {
+    height = settings.value("WidgetHeight").toInt();
+  }
 
   // Ridiculous backwards parenting method to get around Maemo Widget bug:
   webcamImage = new FetCamImage();
@@ -33,12 +71,23 @@ FetWidgetUIForm::FetWidgetUIForm(
     this,
     SLOT(setWebcamName(QString)));
 
+  connect(
+    webcamImage,
+    SIGNAL(pauseDisplay()),
+    this,
+    SLOT(pauseWidget()));
+
   webcamImage->loadFirstImage();
 }
 
 
 FetWidgetUIForm::~FetWidgetUIForm()
 {
+  QSettings settings("pietrzak.org", "Fettuccine");
+
+  settings.setValue("WidgetWidth", width);
+  settings.setValue("WidgetHeight", height);
+
   if (webcamImage) delete webcamImage;
   if (settingsDialog) delete settingsDialog;
 
@@ -48,7 +97,7 @@ FetWidgetUIForm::~FetWidgetUIForm()
 
 QSize FetWidgetUIForm::sizeHint() const
 {
-  return QSize(320, 200);
+  return QSize(width, height);
 }
 
 
@@ -123,6 +172,14 @@ void FetWidgetUIForm::on_prevButton_clicked()
 }
 
 
+void FetWidgetUIForm::on_statusButton_clicked()
+{
+  webcamImage->exitSleepMode();
+
+  ui->statusButton->hide();
+}
+
+
 void FetWidgetUIForm::showSettingsDialog()
 {
   settingsDialog->exec();
@@ -175,3 +232,11 @@ void FetWidgetUIForm::displayTitle(
     ui->nameLabel->hide();
   }
 }
+
+
+void FetWidgetUIForm::pauseWidget()
+{
+  webcamImage->enterSleepMode();
+
+  ui->statusButton->show();
+}
